// -------------------------------------------------------------------------------- 
//!  @file  common.c                                                                
//!  共通処理・データのソースファイル                                               
//                                                                                  
//   Copyright (C) 2021 lsluk, All rights reserved.                                 
//   http://vdmgr.g2.xrea.com/index.html                                            
// -------------------------------------------------------------------------------- 
#include "common.h"
#include "polygon.h"


static dma_channel_config			m_dmaConfig ;					//!< DMA チャンネルコンフィグ

GLOBALVAR							g ;								//!< グローバル変数


// -------------------------------------------------------------------------------- 
//!  初期化                                                                         
//!                                                                                 
//!  @return なし                                                                   
// -------------------------------------------------------------------------------- 
void common_init( void )
{
	int		angle ;

	// 初期化
	setMemory(&g, 0, sizeof( g )) ;

	// サインデータ作成
	for(angle=0; angle<256; angle++) {
		g.sinData[angle] = (int16_t )(sin( M_PI / (256 / 2) * angle ) * 256.) ;
	}

	polygon_init() ;

	vga_init() ;
	pad_init() ;
	vga_start() ;
}


// -------------------------------------------------------------------------------- 
//!  メモリ設定                                                                     
//!                                                                                 
//!  @param [out] s  設定アドレス                                                   
//!  @param [in]  c  設定値                                                         
//!  @param [in]  n  設定サイズ                                                     
//!  @return         なし                                                           
// -------------------------------------------------------------------------------- 
void RAMFUNC setMemory(void *s, uint8_t c, int n)
{
	uint8_t		*p_dest ;
	uint32_t	value ;

	// 32 バイト以下ならバイト単位
	if( n < 32 ) {
		channel_config_set_transfer_data_size(&m_dmaConfig, DMA_SIZE_8) ;
		channel_config_set_read_increment(&m_dmaConfig, false) ;
		dma_channel_configure(DMACH, &m_dmaConfig, s, &c, n, true) ;
		dma_channel_wait_for_finish_blocking( DMACH ) ;
		return ;
	}

	value  = ((uint32_t )c << 8) | c ;
	value |= (value << 16) ;

	p_dest = (uint8_t *)s ;

	// 端数
	if( (uintptr_t )p_dest & 3 ) {
		if( (uintptr_t )p_dest & 1 ) {
			*p_dest++ = (uint8_t )value ;
			n-- ;
		}

		if( (uintptr_t )p_dest & 2 ) {
			*(uint16_t *)p_dest = (uint16_t )value ;
			p_dest             += 2 ;
			n                  -= 2 ;
		}
	}

	channel_config_set_transfer_data_size(&m_dmaConfig, DMA_SIZE_32) ;
	channel_config_set_read_increment(&m_dmaConfig, false) ;
	dma_channel_configure(DMACH, &m_dmaConfig, p_dest, &value, n/4, true) ;

	// 端数
	if( n & 3 ) {
		p_dest += (n & ~3) ;

		if( n & 2 ) {
			*(uint16_t *)p_dest = (uint16_t )value ;
			p_dest             += 2 ;
		}

		if( n & 1 ) {
			*p_dest++ = (uint8_t )value ;
		}
	}

	dma_channel_wait_for_finish_blocking( DMACH ) ;
}


// -------------------------------------------------------------------------------- 
//!  メモリコピー                                                                   
//!  コピー元に ROM 領域の指定禁止                                                  
//!                                                                                 
//!  @param [out] d  コピー先アドレス                                               
//!  @param [in]  s  コピー元アドレス                                               
//!  @param [in]  n  コピーサイズ                                                   
//!  @return         なし                                                           
// -------------------------------------------------------------------------------- 
void RAMFUNC copyMemory(void *d, void const *s, int n)
{
	uint8_t		const *p_srce ;
	uint8_t		*p_dest ;
	int			align ;

	// 32 バイト以下ならバイト単位
	if( n < 32 ) {
		channel_config_set_transfer_data_size(&m_dmaConfig, DMA_SIZE_8) ;
		channel_config_set_read_increment(&m_dmaConfig, true) ;
		dma_channel_configure(DMACH, &m_dmaConfig, d, s, n, true) ;
		dma_channel_wait_for_finish_blocking( DMACH ) ;
		return ;
	}

	p_srce = (uint8_t const *)s ;
	p_dest = (uint8_t       *)d ;
	align  = ((uintptr_t )p_srce ^ (uintptr_t )p_dest) & 3 ;

	if( !align ) {			// ４バイト単位

		if( (uintptr_t )p_dest & 1 ) {
			*p_dest++ = *p_srce++ ;
			n-- ;
		}

		if( (uintptr_t )p_dest & 2 ) {
			*(uint16_t *)p_dest = *(uint16_t const *)p_srce ;
			p_srce             += 2 ;
			p_dest             += 2 ;
			n                  += 2 ;
		}

		channel_config_set_transfer_data_size(&m_dmaConfig, DMA_SIZE_32) ;
		channel_config_set_read_increment(&m_dmaConfig, true) ;
		dma_channel_configure(DMACH, &m_dmaConfig, p_dest, p_srce, n/4, true) ;

		if( n & 3 ) {
			p_srce += (n & ~3) ;
			p_dest += (n & ~3) ;

			if( n & 2 ) {
				*(uint16_t *)p_dest = *(uint16_t const *)p_srce ;
				p_srce             += 2 ;
				p_dest             += 2 ;
			}

			if( n & 1 ) {
				*p_dest = *p_srce ;
			}
		}

	} else
	if( align == 2 ) {		// ２バイト単位

		if( (uintptr_t )p_dest & 1 ) {
			*p_dest++ = *p_srce++ ;
			n-- ;
		}

		channel_config_set_transfer_data_size(&m_dmaConfig, DMA_SIZE_16) ;
		channel_config_set_read_increment(&m_dmaConfig, true) ;
		dma_channel_configure(DMACH, &m_dmaConfig, p_dest, p_srce, n/2, true) ;

		if( n & 1 ) {
			p_srce += (n & ~1) ;
			p_dest += (n & ~1) ;

			*p_dest = *p_srce ;
		}

	} else {				// １バイト単位

		channel_config_set_transfer_data_size(&m_dmaConfig, DMA_SIZE_8) ;
		channel_config_set_read_increment(&m_dmaConfig, true) ;
		dma_channel_configure(DMACH, &m_dmaConfig, p_dest, p_srce, n, true) ;

	}

	dma_channel_wait_for_finish_blocking( DMACH ) ;
}
