// -------------------------------------------------------------------------------- 
//!  @file  common.h                                                                
//!  共通処理・データのヘッダファイル                                               
//                                                                                  
//   Copyright (C) 2021 lsluk, All rights reserved.                                 
//   http://vdmgr.g2.xrea.com/index.html                                            
// -------------------------------------------------------------------------------- 
#ifndef __COMMON_H__INCLUDED__
#define __COMMON_H__INCLUDED__


#include <stdio.h>
#include <stddef.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <pico/stdlib.h>
#include "config.h"
#include "libvga/vga/vga.h"
#include "libvga/pad/pad.h"

#ifdef  DEBUG
#define DBGASSERT( e )				if( !(e) ) { printf("******** %s(%d)\n", __func__, __LINE__) ;  while( 1 ) {} }
																	//!< デバッグ用アサート
#define DBGTRACE(fmt, ...)			printf(fmt, __VA_ARGS__)		//!< デバッグ用トレース
#else
#define DBGASSERT( e )												//!< デバッグ用アサート
#define DBGTRACE(fmt, ...)											//!< デバッグ用トレース
#endif//DEBUG

#define MEMBERSTRUCT(_s, _m, _p)	((_s *)((uintptr_t )(_p) - (uintptr_t )&((_s *)0)->_m))
																	//!< メンバーアドレスから構造体の先頭アドレスを取得

#define __enable_irq()				__asm volatile ("cpsie i")		//!< 割り込み許可
#define __disable_irq()				__asm volatile ("cpsid i")		//!< 割り込み禁止

typedef int32_t						int24f8_t ;						//!< 固定小数点（整数部 24bit、小数部 8bit）


	enum {
		DMACH						=	2,							//!< 共通 DMA チャンネル
		DMACH_POLYGON				=	3,							//!< ポリゴン DMA チャンネル
		FP_ONE_						=	8,							//!< 固定小数点１ビット数
		FP_ONE						=	1 << FP_ONE_,				//!< 固定小数点１
		SCREEN_WIDTH				=	320,						//!< 画面幅
		SCREEN_HEIGHT				=	240,						//!< 画面高さ
	} ;

	//! 点位置（16bit）
	typedef struct {
		int16_t						x ;								//!< ｘ座標
		int16_t						y ;								//!< ｙ座標
	} POINT16 ;

	//! グローバル変数
	typedef struct {
		int16_t						sinData[256] ;					//!< サインデータ

		//! VRAM
		struct {
#if     (BITMAP_MODE==4)
			uint32_t				bitmap[2][SCREEN_HEIGHT][SCREEN_WIDTH/(4*2)] ;
																	//!< ビットマップ
#else
			uint32_t				bitmap[2][SCREEN_HEIGHT][SCREEN_WIDTH/4] ;
																	//!< ビットマップ
#endif//(BITMAP_MODE==4)
			uint16_t				txt[VGA_TXT_HEIGHT_COUNT][VGA_TXT_WIDTH_COUNT] ;
																	//!< テキストデータ
		} vram ;

		dma_channel_config			dmaConfigPolygon ;				//!< ポリゴン用 DMA チャンネルコンフィグ
		uint8_t						*p_bitmap ;						//!< カレントビットマップ
	} GLOBALVAR ;


extern GLOBALVAR					g ;								//!< グローバル変数


extern void common_init( void ) ;
extern void RAMFUNC setMemory(void *s, uint8_t c, int n) ;
extern void RAMFUNC copyMemory(void *d, void const *s, int n) ;


// -------------------------------------------------------------------------------- 
//!  サインを取得                                                                   
//!                                                                                 
//!  @param [in] angle  角度                                                        
//!  @return            固定小数のサイン値                                          
// -------------------------------------------------------------------------------- 
static inline int24f8_t getSin( uint8_t angle )
{
	return (int24f8_t )g.sinData[angle] ;
}


// -------------------------------------------------------------------------------- 
//!  コサインを取得                                                                 
//!                                                                                 
//!  @param [in] angle  角度                                                        
//!  @return            固定小数のコサイン値                                        
// -------------------------------------------------------------------------------- 
static inline int24f8_t getCos( uint8_t angle )
{
	return (int24f8_t )g.sinData[(uint8_t )(angle+64)] ;
}


#endif//__COMMON_H__INCLUDED__
